/*****************************************************************************************************************/
/*   Power Plant Model Validation - PPMV Initialization                                                          */
/*   Bonneville Power Administration 										 */
/*   Program functions:                                                                                          */
/*   - simulate distrbanace play-in                                                                              */
/*                                                                                                               */
/*   Original Version - July 26, 2007                                                                            */
/*   Rev. 1A - April 14, 2011                                                                                    */ 
/*   Rev. 1B - November 1, 2011                                                                                  */ 
/*   Need PSLF 17.07                                                                                             */
/*                                                                                                               */
/*****************************************************************************************************************/

dim *des1[10][120], *inp1[10][120], *str_ret[1][120], *input_line[2][256]
dim *FileName[1][256], *FileLog[1][256]

dim *DydFile[2][256], *SavFile[1][256], *PlotFile[1][256], *DataFile[2][256]

define TIMEMAX  400

/*************************************************************************/

*FileName[0] = "Validation.txt"
*FileLog[0]  = "Validation.log"
*DydFile[0]  = "GEN_GAS_r1.dyd"
*SavFile[0]  = "GEN_GAS.sav"


@ret = setinput(*FileName[0])
if (@ret < 0 )
	logterm("Configuration file ^",*FileName[0],"^ is not found. Run is terminated.<")
	end
endif

@ret = openlog(*FileLog[0])
if (@ret < 0 )
	logterm("Configuration file ^",*FileLog[0],"^ cannot be opened. Run is terminated.<")
	end
endif


logprint(*FileLog[0],"PPMV_PLAY-IN: ")
*str_ret[0] = date(*FileLog[0])
logprint(*FileLog[0],"<<")


for @ix = 0 to 10000
	/* Read the configuration file */
	@ret = inline(*FileName[0], *input_line[0])
	if ( @ret < 0 )
		quitfor
	elseif (toupper(getstr(*input_line[0],0,1))="#")
		/* Comment Line */
		logprint(*FileLog[0], *input_line[0],"<")
	else
		/* Study Run */
		@ret = inbuf(*input_line[0], *DataFile[0], @V0, @P0, @Q0 )

		/* check for the flag */
		*PlotFile[0] = *DataFile[0]  +"_r1.chf"
		*DataFile[0] = *DataFile[0] + ".csv"

		@ret = getf(*SavFile[0])

		/* Slack Bus */
		@SlackBus = 1
		$SlackID  ="1"
		@MW_min = 10
		@gen_inf_id = rec_index(1,3,@SlackBus,0,$SlackID,0,1)
		@bus_inf = bix(@SlackBus)
		busd[@bus_inf].vsched =  @V0 / 500

		/* Setting generator reactive limits to actulas */
		for @j2 = 0 to casepar[0].ngen - 1
			if ( @j2 ~= @gen_inf_id  )           /* not an infinite bus */
				gens[@j2].pgen = @P0
				gens[@j2].qgen = @Q0
				gens[@j2].qmax = gens[@j2].qgen
				gens[@j2].qmin = gens[@j2].qgen
			endif
		next

		/* Mini-State Estimation" to match boundary conditions */		
		@ret = ordr()
		@ret = soln()
		for @niter = 0 to 50
			@delta_p  = gens[@gen_inf_id].pgen + @P0
			@delta_q  = gens[@gen_inf_id].qgen + @Q0
			if( ( abs(@delta_p) < 0.1) and ( abs(@delta_q) < 0.1 )  )
				logterm("Successful Solution !")
				quitfor
			else
				@gen_capacity = 0
				@gen_number   = 0
				for @j2 = 0 to casepar[0].ngen - 1
					if ( @j2 ~= @gen_inf_id  )           /* not an infinite bus */
						if ( gens[@j2].st > 0 )      /* on-line */
							@gen_capacity = @gen_capacity + gens[@j2].pmax
							@gen_number   = @gen_number + 1
						endif
					endif
				next
				/* Distribute Real Power */
				@delta_p = gens[@gen_inf_id].pgen + @P0
				for @j2 = 0 to casepar[0].ngen - 1
					if ( @j2 ~= @gen_inf_id  )           /* not an infinite bus */
						if ( gens[@j2].st > 0 )      /* on-line */
							gens[@j2].pgen = gens[@j2].pgen + @delta_p * gens[@j2].pmax / @gen_capacity
						endif
					endif
				next
				/* Distribute Reactive Power */
				@delta_q = gens[@gen_inf_id].qgen + @Q0
				for @j2 = 0 to casepar[0].ngen - 1
					if ( @j2 ~= @gen_inf_id  )           /* not an infinite bus */
						if ( gens[@j2].st > 0 )      /* on-line */
							gens[@j2].qmax = gens[@j2].qmax + @delta_q * gens[@j2].pmax / @gen_capacity
							gens[@j2].qmin = gens[@j2].qmax
						endif
					endif
				next
			endif		
			logterm("Solution:  Delta P = ", @delta_p, "   Delta Q =", @delta_q,"<")
			@ret = soln()
		next /* niter = 0 to 50 */

		@ret = soln()


		@ret = psds()
		@ret = rdyd(*DydFile[0], "temp.log", 1, 1, 1)
		if (@ret < 0 )
			logterm("DYD File  ", *DydFile[0], " not found. <")
			logprint(*FileLog[0], "DYD File  ", *DydFile[0], " not found. <")
			goto continue
		endif 

		@ret = setinput(*DataFile[0])
		if (@ret < 0 )
			logterm("Disturbance Data File  ", *DataFile[0], " not found. <")
			logprint(*FileLog[0], "Disturbance Data File  ", *DataFile[0], " not found. <")
			goto continue
		else
			for @iy = 1 to 8
				@ret = inline(*DataFile[0], *input_line[0])
			next
			for @jx = 1 to 100000
				@ret = inline(*DataFile[0], *input_line[0])
				if (@ret < 0 )
					quitfor
				else
					@ret = inbuf(*input_line[0], @timesim, @voltage, @frequency, @out1, @out2)
				endif
			next
			@timestop = @timesim
		endif 
		@ret = close(*DataFile[0])


		@zz = rpli(*DataFile[0], 9999, 1, 0 )
		@zz = plsc()

		$repfile = "temp.rep"
		@ret = init(*PlotFile[0], $repfile, "1","1")
		if (@ret < 0 )
			logterm("Initialization Error. <")
			logprint(*FileLog[0], "Initialization Error. <")
			goto continue
		endif 

		dypar[0].delt       = 1/60/4                 
		dypar[0].nplot      = 8                
		dypar[0].nprint     = 8                
		dypar[0].nscreen    = 1000                

		dypar[0].tpause    =  @timestop
		@ret = run()

		if (@ret < 0 )
			logterm("Play-in Run Failed. <")
			logprint(*FileLog[0], "Play-in Run Failed. <")
		else	
			logterm("Play-in Run Complete. <")
			logprint(*FileLog[0], "Play-in Run Complete. <")
		endif

		@ret = dsst()


	endif


label continue


next /* @ix = 0 to 10000 */ 

@ret = close(*FileName[0])
@ret = close(*FileLog[0])


/*******************************************************************************************************************/

